{ Unit:            hashtbl.pas
  Autor:           Sascha Nemecek
  Matrikelnr.:     9825815
  Erstellt am:     09.04.1999
  Letzte nderung: 18.04.1999

  Kurzbeschreibung:
  Ermglicht es in Pascal mit Hashtables zu arbeiten. Dafr stehen
  folgende Methoden zu Verfgung:

  Public:
     find (key):    Sucht nach dem vorkommen eines Schlssels und liefert
                    den Inhalt (Pointer) zurck.
     insert (key):  Fgt ein neues Element in die Hashtable ein.
     delete (key):  Lscht ein Element in der Hashtable.

  Privat:
     h (key):       Liefert den Index des anzusprechenden Arrays.

  Die Hashtable wurde als extern verkettete Hashtable mit 9
  Feldern realisiert.
  Sie ist so konzipiert, das in ihr nur verweise auf beliebige Objekte
  gespeichert werden knnen. Sie dient lediglich zur schnellern
  Auffindung von Objekten. }


UNIT hashtbl;



INTERFACE


{ Definition der Konstanten.}
CONST max = 9;


{ Typendefinitionen.}
TYPE

{ Definition eines Hashknotens }
hashKnotenPtr = ^hashKnoten;
hashKnoten = RECORD
   key: String;
   inhalt: pointer;
   next: hashKnotenPtr;
END;


{ Definition der Hashtable }
hashPtr = ^hash;
hash = OBJECT
   CONSTRUCTOR Init;
   PROCEDURE Insert (key: string; p: pointer);
   PROCEDURE Delete (key: string);

   { Testfunktion. }
   PROCEDURE ListAllKeys;

   FUNCTION Find (key: string) : pointer;
   DESTRUCTOR Done;

  PRIVATE
   hashTab: ARRAY [0..max] of hashKnotenPtr;
   FUNCTION h (key: string): integer;
END;



IMPLEMENTATION


{ Constructor Init.
  Setzt alle Anfangswerte. }
CONSTRUCTOR hash.Init;
VAR i:integer;
BEGIN
   FOR i:=0 TO max DO
      hashTab[i] := NIL;
END;



{ Prozedur Insert.
  Fgt eine neues Element in die Hashtable ein.
  @param key Schlssel des zu Speichernden Pointers.
  @param p Der zu speichernde Pointer. }
PROCEDURE hash.Insert (key: string; p: pointer);
VAR ind: integer;
    hilf: hashKnotenPtr;
BEGIN
   ind := h (key);
   new (hilf);
   hilf^.key := key;
   hilf^.inhalt := p;
   hilf^.next := hashTab [ind];
   hashTab [ind] := hilf;
END;



{ Prozedure Delete.
  Lscht ein Element aus der Hashtable.
  Wenn der gewnschte Schlssel nicht gefunden wird wird kein
  Fehler gemeldet.
  @param key Schlssel des zu lschenden Elements. }
PROCEDURE hash.Delete (key: string);
VAR h1, h2: hashKnotenPtr;
BEGIN
   h1 := hashTab [h (key)];
   h2 := NIL;
   WHILE (h1 <> NIL) AND (h1^.key <> key) DO BEGIN
      h2 := h1;
      h1 := h1^.next;
   END;
   IF h1 <> NIL THEN BEGIN
      IF h2 <> NIL THEN
         h2^.next := h1^.next
      ELSE
         hashTab [h (key)] := h1^.next;
      dispose (h1);
   END;
END;



{ Prozedur ListAllKeys.
  Gibt alle Schlssel am Bildschirm aus. Diese Methode dient nur
  Testzwecken. }
PROCEDURE hash.ListAllKeys;
VAR hilf: hashKnotenPtr;
    i: integer;
BEGIN
   FOR i:=0 TO max DO BEGIN
      hilf := hashTab [i];

      WHILE hilf <> NIL DO BEGIN
         writeln (hilf^.key);
         hilf := hilf^.next;
      END;
   END;
END;



{ Function Find.
  Sucht nach einem Element in der Hasttable und liefert den Pointer
  zurck auf den das Element zeigt.
  @param key Schlssel des zu suchende Eintrag.
  @return Gibt den Inhalt (Pointer) des Eintrags zurck. }
FUNCTION hash.Find (key: string) : pointer;
VAR h1: hashKnotenPtr;
BEGIN
   h1 := hashTab [h (key)];
   WHILE (h1 <> NIL) AND (h1^.key <> key) DO
      h1 := h1^.next;
   IF h1 <> NIL THEN
      Find := h1^.inhalt
   ELSE
      Find := NIL;
END;



{ Funktion h.
  Die Hashfunktion, berechnet den Arrayindex, in dem der Key gespeichert
  werden soll.
  @param key Schlssel des zu speichernden Eintrags.
  @return Index in dem der Eintrag gespeichert werden soll. }
FUNCTION hash.h (key: string) : integer;
VAR i, index: integer;
BEGIN
   index := 0;
   for i := 0 to length (key) DO inc (index, ord (key[i]) * i);
   h := index MOD (max + 1);
END;



{ DESTRUCTOR Done.
  Lscht alle Elemente in der Hashtable. }
DESTRUCTOR hash.Done;
VAR i: integer;
    hilf: hashKnotenPtr;
BEGIN
   FOR i:=0 TO max DO
      WHILE hashTab [i] <> NIL DO BEGIN
         hilf := hashTab [i];
         hashTab [i] := hilf^.next;
         dispose (hilf);
      END;
END;



begin
end.